interface MetaData {
    start: {
        line: number;
        column: number;
        offset: number;
    };
    end: {
        line: number;
        column: number;
        offset: number;
    };
}

declare class Error {
    name: string;
    message: string;
    index: number;
    lineNumber: number;
    column: number;
    description: string;
    constructor(message: string);
}

interface IRegExpFlags {
    value: string;
    literal: string;
}

/**
 * RawToken is converted into TokenEntry by the parser.ts convertToken() function.
 * Hence, this could be called a ParserToken.
 */
interface TokenEntry {
    type?: string | number | 'BlockComment' | 'LineComment';
    value?: number | string;
    lineNumber?: number;
    lineStart?: number;
    literal?: IRegExpFlags;
    octal?: boolean;
    regex?: {
        pattern: string;
        flags: string;
    };
    start?: number;
    startLineNumber?: number;
    startLineStart?: number;
    end?: number;
    loc?: SourceLocation$1;
    range?: [number, number];
}

interface Position$1 {
    line: number;
    column: number;
}
interface SourceLocation$1 {
    start: Position$1;
    end: Partial<Position$1>;
    source?: string;
}

/**
 * TODO: Should be part of the tree?
 */
interface Comment {
    /**
     *
     */
    type: 'Line' | 'Block';
    value: string;
    range?: [number, number];
    loc?: SourceLocation$1;
    extendedRange?: [number, number];
}

declare type StatementType = string;

declare type ArgumentListElement = Expression | SpreadElement;
declare type ArrayExpressionElement = Expression | SpreadElement | null;
declare type ArrayPatternElement = AssignmentPattern | BindingIdentifier | BindingPattern | RestElement | null;
declare type BindingPattern = ArrayPattern | ObjectPattern;
declare type BindingIdentifier = Identifier;
declare type Declaration = AsyncFunctionDeclaration | ClassDeclaration | ExportDeclaration | FunctionDeclaration | ImportDeclaration | VariableDeclaration;
declare type ExportableDefaultDeclaration = BindingIdentifier | BindingPattern | ClassDeclaration | Expression | FunctionDeclaration;
declare type ExportableNamedDeclaration = AsyncFunctionDeclaration | ClassDeclaration | FunctionDeclaration | VariableDeclaration;
declare type ExportDeclaration = ExportAllDeclaration | ExportDefaultDeclaration | ExportNamedDeclaration;
declare type Expression = ArrayExpression | ArrowFunctionExpression | AssignmentExpression | AsyncArrowFunctionExpression | AsyncFunctionExpression | AwaitExpression | BinaryExpression | CallExpression | ClassExpression | ComputedMemberExpression | ConditionalExpression | Identifier | FunctionExpression | Literal | NewExpression | ObjectExpression | RegexLiteral | SequenceExpression | SpreadElement | StaticMemberExpression | TaggedTemplateExpression | ThisExpression | UnaryExpression | UpdateExpression | YieldExpression;
declare type FunctionParameter = AssignmentPattern | BindingIdentifier | BindingPattern;
declare type ImportDeclarationSpecifier = ImportDefaultSpecifier | ImportNamespaceSpecifier | ImportSpecifier;
declare type ObjectExpressionProperty = Property | SpreadProperty;
declare type ObjectPatternProperty = Property | RestProperty;
declare type PatternParam = ArrayPatternElement | Identifier | BindingPattern | Expression | RestProperty | PropertyValue;
declare type Statement = AsyncFunctionDeclaration | BreakStatement | ClassDeclaration | ContinueStatement | DebuggerStatement | DoWhileStatement | EmptyStatement | ExpressionStatement | Directive | ForStatement | ForInStatement | ForOfStatement | FunctionDeclaration | IfStatement | LabeledStatement | ReturnStatement | SwitchStatement | ThrowStatement | TryStatement | VariableDeclaration | WhileStatement | WithStatement;
declare type ReinterpretedExpressionPattern = BindingPattern | AssignmentPattern | RestElement | RestProperty;
declare type PropertyKey = Identifier | Literal | Expression;
declare type PropertyValue = AssignmentPattern | AsyncFunctionExpression | BindingIdentifier | BindingPattern | FunctionExpression | Expression;
declare type StatementListItem = Declaration | Statement;
interface Position {
    /** >= 1 */
    line: number;
    /** >= 0 */
    column: number;
}
interface SourceLocation {
    source?: string | null | undefined;
    start: Position;
    end: Position;
}
interface BaseNodeWithoutComments {
    type: string;
    loc?: SourceLocation | null | undefined;
    range?: [number, number] | undefined;
}
interface BaseNode extends BaseNodeWithoutComments {
    leadingComments: Comment[];
    trailingComments: Comment[];
}
declare class ArrayExpression implements BaseNode {
    type: StatementType;
    readonly elements: ArrayExpressionElement[];
    constructor(elements: ArrayExpressionElement[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ArrayPattern implements BaseNode {
    readonly type: StatementType;
    readonly elements: ArrayPatternElement[];
    constructor(elements: ArrayPatternElement[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ArrowFunctionExpression implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly params: (FunctionParameter | PatternParam)[];
    readonly body: BlockStatement | Expression;
    readonly generator: boolean;
    readonly expression: boolean;
    readonly async: boolean;
    constructor(params: (FunctionParameter | PatternParam)[], body: BlockStatement | Expression, expression: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class AssignmentExpression implements BaseNode {
    type: StatementType;
    operator: string;
    readonly left: Expression;
    readonly right: Expression;
    constructor(operator: string, left: Expression, right: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class AssignmentPattern implements BaseNode {
    readonly type: StatementType;
    readonly left: BindingIdentifier | BindingPattern;
    readonly right: Expression;
    constructor(left: BindingIdentifier | BindingPattern, right: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class AsyncArrowFunctionExpression implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly params: (FunctionParameter | PatternParam)[];
    readonly body: BlockStatement | Expression;
    readonly generator: boolean;
    readonly expression: boolean;
    readonly async: boolean;
    constructor(params: (FunctionParameter | PatternParam)[], body: BlockStatement | Expression, expression: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class AsyncFunctionDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly params: FunctionParameter[];
    readonly body: BlockStatement;
    readonly generator: boolean;
    readonly expression: boolean;
    readonly async: boolean;
    constructor(id: Identifier | null, params: FunctionParameter[], body: BlockStatement);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class AsyncFunctionExpression implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly params: FunctionParameter[];
    readonly body: BlockStatement | Expression;
    readonly generator: boolean;
    readonly expression: boolean;
    readonly async: boolean;
    constructor(id: Identifier | null, params: FunctionParameter[], body: BlockStatement | Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class AwaitExpression implements BaseNode {
    readonly type: StatementType;
    readonly argument: Expression;
    /**
     * Used by escodegen.
     */
    delegate?: unknown;
    constructor(argument: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class BinaryExpression implements BaseNode {
    type: StatementType;
    operator: string;
    readonly left: Expression;
    readonly right: Expression;
    constructor(operator: string, left: Expression, right: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class BlockStatement implements BaseNode {
    readonly type: StatementType;
    readonly body: Statement[] | StatementListItem[];
    constructor(body: Statement[] | StatementListItem[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class BreakStatement implements BaseNode {
    readonly type: StatementType;
    readonly label: Identifier | null;
    constructor(label: Identifier | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class CallExpression implements BaseNode {
    type: string;
    callee: Expression | Import;
    arguments: ArgumentListElement[];
    optional: boolean;
    constructor(callee: Expression | Import, args: ArgumentListElement[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class CatchClause implements BaseNode {
    readonly type: StatementType;
    readonly param: BindingIdentifier | BindingPattern;
    readonly body: BlockStatement;
    /**
     * Used by escodegen.
     */
    guard?: Expression;
    constructor(param: BindingIdentifier | BindingPattern, body: BlockStatement);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ClassBody implements BaseNode {
    readonly type: StatementType;
    readonly body: (Property | MethodDefinition)[];
    constructor(body: (Property | MethodDefinition)[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ClassDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly superClass: Identifier | Expression | null;
    readonly body: ClassBody;
    constructor(id: Identifier | null, superClass: Identifier | Expression | null, body: ClassBody);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ClassExpression implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly superClass: Identifier | Expression | null;
    readonly body: ClassBody;
    constructor(id: Identifier | null, superClass: Identifier | Expression | null, body: ClassBody);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class MemberExpression implements BaseNode {
    readonly type: StatementType;
    readonly computed: boolean;
    readonly object: Expression;
    readonly property: Expression;
    constructor(object: Expression, property: Expression, computed: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ComputedMemberExpression extends MemberExpression {
    constructor(object: Expression, property: Expression);
}
declare class ConditionalExpression implements BaseNode {
    readonly type: StatementType;
    readonly test: Expression;
    readonly consequent: Expression;
    readonly alternate: Expression;
    constructor(test: Expression, consequent: Expression, alternate: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ContinueStatement implements BaseNode {
    readonly type: StatementType;
    readonly label: Identifier | null;
    constructor(label: Identifier | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class DebuggerStatement implements BaseNode {
    readonly type: StatementType;
    constructor();
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class Directive implements BaseNode {
    readonly type: StatementType;
    readonly expression: Expression;
    readonly directive: string;
    /**
     * Used by escodegen.
     */
    raw?: string;
    constructor(expression: Expression, directive: string);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class DoWhileStatement implements BaseNode {
    readonly type: StatementType;
    body: Statement;
    readonly test: Expression;
    constructor(body: Statement, test: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class EmptyStatement implements BaseNode {
    readonly type: StatementType;
    constructor();
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ExportAllDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly source: Literal;
    constructor(source: Literal);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ExportDefaultDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly declaration: ExportableDefaultDeclaration;
    constructor(declaration: ExportableDefaultDeclaration);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ExportNamedDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly declaration: ExportableNamedDeclaration | StatementListItem | null;
    readonly specifiers: ExportSpecifier[];
    readonly source: Literal | null;
    constructor(declaration: ExportableNamedDeclaration | StatementListItem | null, specifiers: ExportSpecifier[], source: Literal | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ExportSpecifier implements BaseNode {
    readonly type: StatementType;
    readonly exported: Identifier;
    readonly local: Identifier;
    constructor(local: Identifier, exported: Identifier);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ExpressionStatement implements BaseNode {
    readonly type: StatementType;
    readonly expression: Expression;
    constructor(expression: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ForInStatement implements BaseNode {
    readonly type: StatementType;
    readonly left: Expression;
    readonly right: Expression;
    readonly body: Statement | Expression;
    readonly each: boolean;
    constructor(left: Expression, right: Expression, body: Statement | Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ForOfStatement implements BaseNode {
    readonly type: StatementType;
    readonly left: Expression;
    readonly right: Expression;
    readonly body: Statement | Expression;
    constructor(left: Expression, right: Expression, body: Statement | Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ForStatement implements BaseNode {
    readonly type: StatementType;
    readonly init: Expression | null;
    readonly test: Expression | null;
    readonly update: Expression | null;
    body: Statement | Expression;
    constructor(init: Expression | null, test: Expression | null, update: Expression | null, body: Statement | Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class FunctionDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly params: FunctionParameter[];
    readonly body: BlockStatement;
    readonly generator: boolean;
    readonly expression: boolean;
    readonly async: boolean;
    constructor(id: Identifier | null, params: FunctionParameter[], body: BlockStatement, generator: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class FunctionExpression implements BaseNode {
    readonly type: StatementType;
    readonly id: Identifier | null;
    readonly params: FunctionParameter[];
    readonly body: BlockStatement | Expression;
    readonly generator: boolean;
    readonly expression: boolean;
    readonly async: boolean;
    constructor(id: Identifier | null, params: FunctionParameter[], body: BlockStatement | Expression, generator: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class Identifier implements BaseNode {
    type: StatementType;
    name: string;
    range: [number, number];
    constructor(name: string | number);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
}
declare class IfStatement implements BaseNode {
    readonly type: StatementType;
    readonly test: Expression;
    readonly consequent: Statement;
    readonly alternate: Statement | null;
    constructor(test: Expression, consequent: Statement, alternate: Statement | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class Import implements BaseNode {
    readonly type: StatementType;
    constructor();
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ImportDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly specifiers: ImportDeclarationSpecifier[];
    readonly source: Literal;
    constructor(specifiers: ImportDeclarationSpecifier[], source: Literal);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ImportDefaultSpecifier implements BaseNode {
    readonly type: StatementType;
    readonly local: Identifier;
    constructor(local: Identifier);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ImportNamespaceSpecifier implements BaseNode {
    readonly type: StatementType;
    readonly local: Identifier;
    constructor(local: Identifier);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ImportSpecifier implements BaseNode {
    readonly type: StatementType;
    readonly local: Identifier;
    readonly imported: Identifier;
    constructor(local: Identifier, imported: Identifier);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class LabeledStatement implements BaseNode {
    readonly type: StatementType;
    readonly label: Identifier;
    readonly body: Statement | ClassDeclaration;
    constructor(label: Identifier, body: Statement | ClassDeclaration);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class Literal implements BaseNode {
    readonly type: StatementType;
    readonly value: boolean | number | string | null;
    readonly raw: string;
    range: [number, number];
    constructor(value: boolean | number | string | null, raw: string);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
}
declare class MethodDefinition implements BaseNode {
    readonly type: StatementType;
    readonly key: Expression | null;
    readonly computed: boolean;
    readonly value: AsyncFunctionExpression | FunctionExpression | null;
    readonly kind: string;
    readonly static: boolean;
    constructor(key: Expression | null, computed: boolean, value: AsyncFunctionExpression | FunctionExpression | null, kind: string, isStatic: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class Module implements BaseNode {
    readonly type: StatementType;
    readonly body: StatementListItem[];
    readonly sourceType: string;
    comments?: Comment[];
    tokens?: TokenEntry[];
    errors?: Error[];
    constructor(body: StatementListItem[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class NewExpression implements BaseNode {
    readonly type: StatementType;
    readonly callee: Expression;
    readonly arguments: ArgumentListElement[];
    constructor(callee: Expression, args: ArgumentListElement[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ObjectExpression implements BaseNode {
    type: StatementType;
    readonly properties: ObjectExpressionProperty[];
    constructor(properties: ObjectExpressionProperty[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ObjectPattern implements BaseNode {
    readonly type: StatementType;
    readonly properties: ObjectPatternProperty[];
    constructor(properties: ObjectPatternProperty[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class Property implements BaseNode {
    readonly type: StatementType;
    readonly key: PropertyKey;
    readonly computed: boolean;
    readonly value: PropertyValue | null;
    readonly kind: string;
    readonly method: boolean;
    readonly shorthand: boolean;
    constructor(kind: 'get' | 'set' | 'init', key: PropertyKey, computed: boolean, value: PropertyValue | null, method: boolean, shorthand: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class RegexLiteral implements BaseNode {
    readonly type: StatementType;
    readonly value: RegExp;
    readonly raw: string;
    readonly regex: {
        pattern: string;
        flags: string;
    };
    constructor(value: RegExp, raw: string, pattern: string, flags: string);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class RestElement implements BaseNode {
    type: StatementType;
    argument: BindingIdentifier | BindingPattern | ReinterpretedExpressionPattern;
    constructor(argument: BindingIdentifier | BindingPattern | ReinterpretedExpressionPattern);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class RestProperty implements BaseNode {
    type: StatementType;
    argument: Expression;
    constructor(argument: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ReturnStatement implements BaseNode {
    readonly type: StatementType;
    readonly argument: Expression | null;
    constructor(argument: Expression | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class Script implements BaseNode {
    readonly type: StatementType;
    readonly body: StatementListItem[];
    readonly sourceType: string;
    comments?: Comment[];
    tokens?: TokenEntry[];
    errors?: Error[];
    constructor(body: StatementListItem[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class SequenceExpression implements BaseNode {
    readonly type: StatementType;
    readonly expressions: Expression[];
    constructor(expressions: Expression[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class SpreadElement implements BaseNode {
    type: StatementType;
    readonly argument: Expression;
    constructor(argument: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class SpreadProperty implements BaseNode {
    readonly type: StatementType;
    readonly argument: Expression;
    constructor(argument: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class StaticMemberExpression implements BaseNode {
    readonly type: StatementType;
    readonly computed: boolean;
    readonly object: Expression;
    readonly property: Expression;
    constructor(object: Expression, property: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class SwitchCase implements BaseNode {
    readonly type: StatementType;
    readonly test: Expression;
    readonly consequent: Statement[] | StatementListItem[];
    constructor(test: Expression, consequent: Statement[] | StatementListItem[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class SwitchStatement implements BaseNode {
    readonly type: StatementType;
    readonly discriminant: Expression;
    readonly cases: SwitchCase[];
    constructor(discriminant: Expression, cases: SwitchCase[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class TaggedTemplateExpression implements BaseNode {
    readonly type: StatementType;
    readonly tag: Expression;
    readonly quasi: TemplateLiteral;
    constructor(tag: Expression, quasi: TemplateLiteral);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
interface TemplateElementValue {
    cooked: string;
    raw: string;
}
declare class TemplateElement implements BaseNode {
    readonly type: StatementType;
    readonly value: TemplateElementValue;
    readonly tail: boolean;
    constructor(value: TemplateElementValue, tail: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class TemplateLiteral implements BaseNode {
    readonly type: StatementType;
    readonly quasis: TemplateElement[];
    readonly expressions: Expression[];
    constructor(quasis: TemplateElement[], expressions: Expression[]);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ThisExpression implements BaseNode {
    readonly type: StatementType;
    constructor();
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class ThrowStatement implements BaseNode {
    readonly type: StatementType;
    readonly argument: Expression;
    constructor(argument: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class TryStatement implements BaseNode {
    readonly type: StatementType;
    readonly block: BlockStatement;
    readonly handler: CatchClause | null;
    readonly finalizer: BlockStatement | null;
    /**
     * Used by escodegen.
     */
    handlers: CatchClause[];
    /**
     * Used by escodegen.
     */
    guardedHandlers: CatchClause[];
    constructor(block: BlockStatement, handler: CatchClause | null, finalizer: BlockStatement | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class UnaryExpression implements BaseNode {
    readonly type: StatementType;
    operator: string;
    readonly argument: Expression;
    readonly prefix: boolean;
    constructor(operator: string, argument: Expression);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class UpdateExpression implements BaseNode {
    readonly type: StatementType;
    operator: string;
    readonly argument: Expression;
    readonly prefix: boolean;
    constructor(operator: string, argument: Expression, prefix: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class VariableDeclaration implements BaseNode {
    readonly type: StatementType;
    readonly declarations: VariableDeclarator[];
    readonly kind: string;
    constructor(declarations: VariableDeclarator[], kind: string);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class VariableDeclarator implements BaseNode {
    readonly type: StatementType;
    readonly id: BindingIdentifier | BindingPattern;
    readonly init: Expression | null;
    constructor(id: BindingIdentifier | BindingPattern, init: Expression | null);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class WhileStatement implements BaseNode {
    readonly type: StatementType;
    readonly test: Expression;
    body: Statement;
    constructor(test: Expression, body: Statement);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class WithStatement implements BaseNode {
    readonly type: StatementType;
    readonly object: Expression;
    readonly body: Statement;
    constructor(object: Expression, body: Statement);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}
declare class YieldExpression implements BaseNode {
    type: StatementType;
    argument: Expression | null;
    delegate: boolean;
    constructor(argument: Expression | null, delegate: boolean);
    leadingComments: Comment[];
    trailingComments: Comment[];
    loc?: SourceLocation;
    range?: [number, number];
}

interface FormatOptions {
    indent?: {
        style: string;
        base: number;
        adjustMultilineComment: boolean;
    };
    newline?: '\n';
    space?: ' ';
    json?: boolean;
    renumber?: boolean;
    hexadecimal?: boolean;
    quotes?: 'single' | 'double';
    escapeless?: boolean;
    compact?: boolean;
    parentheses?: boolean;
    semicolons?: boolean;
    safeConcatenation?: boolean;
    preserveBlankLines?: boolean;
}
interface MozillaOptions {
    comprehensionExpressionStartsWithAssignment: false;
    starlessGenerator: false;
}
interface GenerateOptions {
    parse?: (arg: unknown) => Module | Script;
    comment?: boolean;
    file?: string;
    format?: FormatOptions;
    moz?: MozillaOptions;
    sourceContent?: string;
    sourceMap?: null;
    sourceMapRoot?: null;
    sourceMapWithCode?: false;
    directive?: false;
    raw?: true;
    verbatim?: string | null;
    sourceCode?: null;
}

interface ParseOptions {
    comment?: boolean;
    attachComment?: boolean;
    sourceType?: 'module' | 'script';
    jsx?: boolean;
    /**
     * Determines whether the parser reports range: [start: number, end: number].
     */
    range?: boolean;
    /**
     * Determines whether the parser reports loc: {start:{line:number;column:number}; end:{line:number;column:number}}
     */
    loc?: boolean;
    tokens?: boolean;
    tolerant?: boolean;
    source?: string;
}
interface ParseDelegate {
    (node: TokenEntry, metadata?: MetaData): TokenEntry;
}

/**
 * Provides the MathScript module
 */
/**
 * Determines the transformation action for operators.
 */
interface OperatorTransform {
    /**
     * The nature of the transformation.
     */
    kind: 'rename' | 'function' | 'error';
    /**
     * The transformed name.
     */
    value: string;
}
/**
 *
 */
interface TranspileOptions extends ParseOptions {
    timeout?: number;
    noLoopCheck?: boolean;
    namespace?: string;
    /**
     * The replacements of unary operators that will be made.
     */
    binOp?: {
        [name: string]: OperatorTransform;
    };
    /**
    * The replacements of unary operators that will be made.
    */
    unaryOp?: {
        [name: string]: OperatorTransform;
    };
}
/**
 * This is the function that we export.
 */
declare function jsToMathScript(code: string, transpileOptions?: TranspileOptions, delegate?: ParseDelegate, generateOptions?: GenerateOptions): {
    code: string;
};

export { GenerateOptions, OperatorTransform, TranspileOptions, jsToMathScript };
